from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import math
import commands


class LTVClock(Sensor):
	def __init__(self, hour = "black", minute = "black", second = "red", time="%h:%m%a", date="%M %d, %y"):
		global _; _ = i18n.Translator("ltv-clock-sensor")
		Sensor.__init__(self)
		self._set_config_type("show", TYPE_BOOL, 1)
		self._set_config_type("h",   TYPE_STRING, hour)
		self._set_config_type("m", TYPE_STRING, minute)
		self._set_config_type("s", TYPE_STRING, second)
		self._set_config_type("t", TYPE_STRING, time)
		self._set_config_type("d", TYPE_STRING, date)
		self._set_config_type("timezone", TYPE_STRING, "localtime")
	
		self.__last_date = ""
		self.__last_time = ""
		self.__timezones = [(_("local"), "localtime"), (_("UTC"), "UTC0")] + self.__load_timezones()
		self.__timezone = ""
		self.__offset = 0
		self.__months = [ _("January"), _("February"), _("March"), _("April"), _("May"), _("June"), _("July"), _("August"), _("September"), _("October"), _("November"), _("December") ]
		self.__enabled = [ time != "NULL", date != "NULL" ]
		self._add_timer(0, self.__set_timezone)
		self._add_timer(0, self.tick_tock)
		self._watch_config( self.watch_config )

	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name(_("Clock"))
		configurator.add_title(_("Clock Settings"))
		configurator.add_checkbox(_("_Show second hand"), "show", _("Toggles the seconds on and off"))
		configurator.add_color_selector(_("Hour hand:"), "h", _("Color used for the hour hand"))
		configurator.add_color_selector(_("Minute hand:"), "m", _("Color used for the minute hand"))
		configurator.add_color_selector(_("Second hand:"), "s", _("Color used for the second hand"))
		configurator.add_title(_("Regional Settings"))
		configurator.add_option(_("Timezone"), "timezone", _("The timezone to use"), self.__timezones)
		if self.__enabled[ 0 ] or self.__enabled[ 1 ]:
			configurator.add_title(_("Formatting"))
			if self.__enabled[ 0 ]:
				configurator.add_entry(_("Time:"), "t", _("Time Formatting:\n%h\tHour on a 12 hour clock\n%H\tHour on a 24 hour clock\n%m\tMinutes\n%s\tSeconds\n%a\tam/pm\n%A\tAM/PM"))
			if self.__enabled[ 1 ]:
				configurator.add_entry(_("Date:"), "d", _("Date Formatting:\n%w\tDay of the week\n%m\tMonth as a number\n%M\tMonth as a word\n%d\tDay\n%y\tYear"))
		return configurator
	
	def __rotate(self, x, y, angle):
		cos_a = math.cos(angle)
		sin_a = math.sin(angle)
		rx = x * cos_a - y * sin_a
		ry = y * cos_a + x * sin_a
		return (rx, ry)
	
	def __do_hand(self, points, angle):
		p = ""
		for i in xrange(0, len(points), 2):
			x = points[i]
			y = points[i + 1]
			rx, ry = self.__rotate(x, y, angle)
			p += " %(rx)f %(ry)f" % vars()
		return p
	
	def __load_timezones(self):
		zones = []
		try:
			fd = open("timezones.dat", "r")
			try:
				lines = fd.readlines()
			finally:
				fd.close()
			for l in lines:
				name = l.strip()
				zones.append((name, name))
		except:
			pass
		return zones
	
	def draw(self):
		current_time = time.gmtime(time.time() + self.__offset)
		hour = current_time[3]
		minute = current_time[4]
		second = current_time[5]
		gfx_hour = self.__do_hand([-0.05, -0.0, 0.05, -0.0, 0.05, 0.6, -0.05, 0.6], -(2 * math.pi) / 12 * (hour + (minute / 60.0)))
		gfx_minute = self.__do_hand([-0.05, -0.0, 0.05, -0.0, 0.05, 1.0, -0.05, 1.0], -(2 * math.pi) / 60 * minute)
		if (self._get_config("show")):
			gfx_second = self.__do_hand([-0.025, -0.0, 0.025, -0.0, 0.025, 1.0, -0.025, 1.0], -(2 * math.pi) / 60 * second)
		else:
			gfx_second = ""
		data = self._new_output()
		if self.__enabled[ 1 ]:
			date_ = self.__format_date( current_time[1], current_time[2], current_time[0])
			if date_ != self.__last_date:
				self.__last_date = date_
				data.set("date", date_ )
		if self.__enabled[ 0 ]:
			time_ = self.__format_time( hour, minute, second)
			if time_ != self.__last_time:
				self.__last_time = time_
				data.set("time", time_ )
		data.set("hands", "clear, color " + self._get_config("h") + (", polygon %(gfx_hour)s 1, " % vars()) + "color " + self._get_config("m") + (",polygon %(gfx_minute)s 1," % vars()) + "color " + self._get_config("s") + (",polygon %(gfx_second)s 1" % vars()))
		self._send_output(data)

	def __format_date(self, m, d, y):
		return self._get_config("d").replace("%m",str(m)).replace("%M",self.__months[ m - 1 ]).replace("%d",str(d)).replace("%y",str(y))

	def __format_time(self, hour, minute, second):
		h = "%(hour)02d" % vars()
		m = "%(minute)02d" % vars()
		s = "%(second)02d" % vars()
		if hour < 12 :
			a = "am"
			A = "AM"
		else:
			a = "pm"
			A = "PM"
			hour = hour - 12
		if hour == 0:
			hour = 12
		return self._get_config("t").replace("%H",h).replace("%h",str( hour )).replace("%m",m).replace("%s",s).replace("%a",a).replace("%A",A)

	def __set_timezone(self):
		timezone = self._get_config("timezone")
		if timezone != self.__timezone:
			self.__timezone = timezone
			if timezone == "localtime":
				offset = commands.getoutput("date +%z")
			else:
				offset = commands.getoutput("TZ=\"%(timezone)s\" date +%%z" % vars())
			sign = (offset[0] == "+") and 1 or -1
			hours = int(offset[1:3])
			mins = int(offset[3:5])
			self.__offset = sign * (mins * 60 + hours * 3600)


	def tick_tock(self):
		self.draw()
		self._add_timer(1000, self.tick_tock)

	def watch_config(self,key,value):
		if key == "timezone" :
			self.__set_timezone()
		self.draw()
	
def new_sensor(args): return apply(LTVClock, args)
